<?php

namespace WPEP\Admin;

use WPEP\Controller as WPEP_Controller;

class Controller {

  /**
   * @var Coordinator\AddOnsSection
   */
  public $addOnsSection;
  /**
   * @var Coordinator\Assessment
   */
  public $assessment;
  /**
   * @var Coordinator\Assets
   */
  public $assets;
  /**
   * @var Coordinator\Course
   */
  public $course;
  /**
   * @var Coordinator\Builder
   */
  public $builder;
  /**
   * @var Coordinator\ContentLibrary
   */
  public $contentLibrary;
  /**
   * @var Coordinator\EBook
   */
  public $eBook;
  /**
   * @var Coordinator\Video
   */
  public $video;
  /**
   * @var Coordinator\Offer
   */
  public $offer;
  /**
   * @var Coordinator\PlatformHealth
   */
  public $platformHealth;
  /**
   * @var Coordinator\Post
   */
  public $post;
  /**
   * @var Coordinator\License
   */
  public $license;
  /**
   * @var Coordinator\MetaBoxes
   */
  public $metaBoxes;
  /**
   * @var Coordinator\ShortCodeBuilder
   */
  public $shortCodeBuilder;
  /**
   * @var Coordinator\Statistics
   */
  public $statistics;
  /**
   * @var Coordinator\Settings
   */
  public $settings;
  /**
   * @var Coordinator\EmailNotification
   */
  public $emailNotification;
  /**
   * @var Coordinator\LearningCenter
   */
  public $learningCenter;

  /**
   * @return Controller
   */
  public function __construct() {
    return $this;
  }

  public function dispatch() {
    if( !is_admin() )
      return false;

    $this->_define();
    $this->_setup();

    add_filter( 'plugin_action_links_' . plugin_basename( WPEP_BASE_FILE_PATH ), [ $this, 'action_links' ] );
    add_filter( 'user_row_actions', [ $this, 'user_row_actions' ], 10, 2 );

    add_action( 'wpep_admin_download_table', [ $this, 'download_table_csv' ], 10, 3 );

    add_action( 'create_course_category', [ $this, '_term_created' ] );

    if( wpep_has_embed_support() )
      add_filter( WPEP_TAXONOMY_CATEGORY . "_row_actions", [ $this, 'course_category_table_row_actions' ], 15, 2 );

    if( wpep_get_setting_flag( 'setup-rewrite-rules-flushed' ) == false )
      add_action( 'init', [ $this, '_activation_init' ], 5 );

    return true;
  }

  private function _define() {
    $this->addOnsSection = new Coordinator\AddOnsSection();
    $this->assessment    = new Coordinator\Assessment();
    $this->assets        = new Coordinator\Assets();
    $this->course        = new Coordinator\Course();
    $this->builder       = new Coordinator\Builder();
    $this->contentLibrary= new Coordinator\ContentLibrary();
    $this->eBook         = new Coordinator\EBook();
    $this->video         = new Coordinator\Video();
    $this->offer         = new Coordinator\Offer();
    $this->platformHealth= new Coordinator\PlatformHealth();
    $this->post          = new Coordinator\Post();
    $this->license       = new Coordinator\License();
    $this->metaBoxes     = new Coordinator\MetaBoxes();
    $this->settings      = new Coordinator\Settings();
    $this->shortCodeBuilder  = new Coordinator\ShortCodeBuilder();
    $this->statistics        = new Coordinator\Statistics();
    $this->emailNotification = new Coordinator\EmailNotification();
    $this->learningCenter    = new Coordinator\LearningCenter();
  }

  private function _setup() {
    $this->addOnsSection->setup();
    $this->assessment->setup();
    $this->assets->setup();
    $this->course->setup();
    $this->builder->setup();
    $this->contentLibrary->setup();
    $this->eBook->setup();
    $this->video->setup();
    $this->offer->setup();
    $this->platformHealth->setup();
    $this->post->setup();
    $this->license->setup();
    $this->metaBoxes->setup();
    $this->settings->setup();
    $this->shortCodeBuilder->setup();
    $this->statistics->setup();
    $this->emailNotification->setup();
    $this->learningCenter->setup();
  }

  public function _activation_init() {
    try {
      flush_rewrite_rules();

      WPEP_Controller::instance()->capabilities->register_caps_and_roles();

      $this->platformHealth->delete_error_transient_storage();
    } catch ( \Exception $exception ) {

    }

    wpep_set_setting( 'setup-rewrite-rules-flushed', 1 );
  }

  public function _term_created( $term_id ) {
    update_term_meta( $term_id, 'wpep_order', intval( get_term_meta( $term_id, 'wpep_order', true  ) ) );
  }

  public function action_links( $links ) {
    $links[] = '<a href="'. esc_url( get_admin_url(null, 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-settings') ) .'">' . __( "Settings", 'wpep' )  . '</a>';
    return $links;
  }

  public function user_row_actions( $actions, $user_object ) {
    $actions[ 'wpep_course_statistics' ] = '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'user_id' => $user_object->ID ] ) . '">' . __( "Course Statistics", "wpep" ) . '</a>';

    if( wpep_assessments_enabled() )
      $actions[ 'wpep_assessment_statistics' ] = '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'user_id' => $user_object->ID ] ) . '">' . __( "Assessment Statistics", "wpep" ) . '</a>';

    if( wpep_public_profiles_enabled() )
      $actions[ 'wpep_public_profile' ] = '<a href="' . wpep_user_profile_public_link( $user_object ). '">' . __( "WPEP Public Profile", "wpep" ) . '</a>';

    return $actions;
  }

  public function course_category_table_row_actions( $actions, $term ) {
    $actions['wpep_external_embed'] = wpep_get_external_page_embed_copy_button(
      $term->term_id, get_term_link( $term ), 'wpep-external-embed-text aria-button-if-js'
    );

    return $actions;
  }

  public function download_table_csv( $name = 'download',
                                      $head_row_data = [],
                                      $body_row_list = [] ) {
    if( empty( $head_row_data ) || empty( $body_row_list ) )
      return;

    $name          = sanitize_title( $name );
    $head_row_data = $this->_get_csv_head( $head_row_data );

    header('Content-Type: application/csv');
    header('Content-Disposition: attachment; filename="'. $name .'.csv";');

    $output_stream = fopen('php://output', 'w');

    fputcsv( $output_stream, $head_row_data , ';' );

    foreach ( $body_row_list as $body_row_item ) {
      fputcsv( $output_stream, $this->_get_csv_body_row( $body_row_item, $head_row_data ), ';' );
    }

    fclose( $output_stream );

    exit;
  }

  private function _get_csv_head( $head_row_data ) {
    foreach( $head_row_data as $key => $value ) {
      if( is_string( $value ) )
        continue;

      if( isset( $value['type'] ) && $value['type'] == 'link-item' ) {
        unset( $head_row_data[ $key ] );
        continue;
      }

      $head_row_data[ $key ] = (string)( isset( $value['value_raw'] ) ? $value['value_raw'] : $value['value'] );
    }

    return $head_row_data;
  }

  private function _get_csv_body_row( $body_row_data, $head_row_data ) {
    $resp = [];

    foreach( $head_row_data as $head_row_key => $head_row_value ) {
      $current_body_row_data = $body_row_data[ $head_row_key ];

      $resp[ $head_row_key ] = (string) ( is_string( $current_body_row_data )
        ? $current_body_row_data
        : ( isset( $current_body_row_data['value_raw'] ) ? $current_body_row_data['value_raw'] : $current_body_row_data['value'] ) );
    }

    return $resp;
  }

}