<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Admin\Container\AssessmentBuilder as WPEP_Container_AssessmentBuilder;
use WPEP\Admin\Container\CourseBuilder as WPEP_Container_CourseBuilder;

/**
 * declare Course class.
 *
 * @since        1.0
 */

class Course {

  use WPEP_Container_AssessmentBuilder, WPEP_Container_CourseBuilder;

  public function setup() {
    add_filter('wpep_meta_boxes', [ $this, '_init_custom_meta_boxes' ] );

    add_action('post_row_actions', [ $this, 'row_actions' ], 10, 2);
    add_action('admin_action_wpep_duplicate_course', [ $this, 'duplicate_run' ] );

    add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
    add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );

    add_action( 'save_post', [ $this, '_save_post_action' ] );
    add_filter( 'wpep_admin_save_post_ajax', [ $this, '_wpep_admin_save_post_ajax' ], 10, 2 );
  }

  public function _save_post_action( $course_post_id ) {
    global $post_type;

    if ( !is_admin() || ( $post_type != WPEP_POST_TYPE_COURSE ) )
      return;

    $entity_id_map = ( isset( $_POST['wpep_entity_id_map'] ) ? $_POST['wpep_entity_id_map'] : [] );

    if( is_string( $entity_id_map ) ) {
      $entity_id_map = json_decode( stripslashes( $entity_id_map ), true );
    }

    if( !is_array( $entity_id_map ) )
      $entity_id_map = [];

    $entity_id_map = $entity_id_map + [
      'section' => [],
      'lesson'  => []
    ];

    $this->_save_post( $course_post_id, $entity_id_map );
  }

  public function _wpep_admin_save_post_ajax( $entity_id_map, $post_id ) {
    return $this->_save_post( $post_id, $entity_id_map );
  }

  public function _save_post( $course_post_id, $entity_id_map = [] ) {
    if( get_post_type( $course_post_id ) != WPEP_POST_TYPE_COURSE )
      return $entity_id_map;

    if( intval( get_post_meta( $course_post_id, 'wpep_order', true ) ) == false )
      update_post_meta( $course_post_id, 'wpep_order', 0 );

    if (isset($_POST['wpep_course_description'])) {
      $course_description = stripcslashes ( $_POST['wpep_course_description'] );

      if( $course_description !== '' )
        update_post_meta( $course_post_id, 'wpep_course_description', $course_description );
      else
        delete_post_meta( $course_post_id, 'wpep_course_description' );
    }

    $entity_id_map = $this->handle_course_save_functionality( $course_post_id, $entity_id_map );
    $entity_id_map = $this->handle_assessment_save_functionality( $course_post_id, $entity_id_map );

    if( isset( $_POST['wpep_current_primary_tab'] ) )
      update_post_meta( $course_post_id, 'wpep_current_primary_tab', $_POST['wpep_current_primary_tab'] );

    return $entity_id_map;
  }

  public function _init_custom_meta_boxes(array $meta_boxes) {
    $course_meta_boxes = [];

    $course_meta_boxes['wpep_course_options']  = [
      'id'         => 'wpep_course_options',
      'title'      => __('General Options', "wpep"),
      'post_types' => [ WPEP_POST_TYPE_COURSE ],
      'context'    => 'side',
      'priority'   => 'low',
      'fields'     => $this->_get_course_options_metabox()
    ];

    if( wpep_get_setting_flag( 'enable-course-tracking-code-option' ) ) {
      $course_meta_boxes['wpep_tracking_code']  = [
        'id'         => 'wpep_tracking_code',
        'title'      => __('Tracking Code Configuration', "wpep"),
        'post_types' => [ WPEP_POST_TYPE_COURSE ],
        'context'    => 'advanced',
        'priority'   => 'low',
        'fields'     => [
          [
            'name'    => __('Tracking Code', "wpep"),
            'id'      => 'wpep_tracking_code_completed',
            'type'    => 'textarea',
            'height'  => '300px;',
            'desc'    => __(
              "This is a custom javascript code or HTML code executed when the student has completed the course.",
              "wpep"
            )
          ],
        ]
      ];
    }

    $course_meta_boxes = apply_filters( "wpep_administration_course_metaboxes", $course_meta_boxes );

    return $meta_boxes + $course_meta_boxes;
  }

  private function _get_course_options_metabox() {
    $course_prerequisites = wpep_utility_map_to_array_assoc(
      get_posts( [
        'post_type'   => WPEP_POST_TYPE_COURSE,
        'orderby'     => 'title',
        'order'       => 'ASC',
        'numberposts' => -1,
        ''
      ] ), 'ID', 'post_title'
    );

    unset( $course_prerequisites[ get_the_ID() ] );

    $wpep_course_options_fields = [];

    if( wpep_has_sid_support() ) {
      $wpep_course_options_fields[ 'wpep_course_identifier' ] = [
        'name'    => __( 'Course Reports Identifier', "wpep" ),
        'id'      => 'wpep_course_identifier',
        'type'    => 'text'
      ];
    }

    $wpep_course_options_fields += [
      'course_type' => [
        'name'    => __('Course Type', "wpep"),
        'desc'    => __('By default a course is an execution plan.', "wpep"),
        'id'      => 'course_type',
        'type'    => 'select',
        'options' => wpep_course_layout_types()
      ],
      'course_theme' => [
        'name'    => __('Course Theme', "wpep"),
        'desc'    => __('This option in not available for shortcode embeds of this course.', "wpep"),
        'id'      => 'course_theme',
        'type'    => 'select',
        'options' => wpep_get_theme_list_as_entity_layout_option()
      ],
      'wpep_standard_course_overview_page'  => [
        'name'    => __( 'Overview Page', "wpep" ),
        'desc'    => '<p>' . __( "Individually chose if this course should have an Index Page / Overview Page", "wpep" ) . '</p>',
        'id'      => 'wpep_standard_course_overview_page',
        'type'    => 'select',
        'options' => [
          'default' => sprintf( __( "Course Type Default ( %s )" ), ( wpep_get_setting_flag( 'course-standard-allow-index-page' ) ? __( 'Enabled', "wpep" ) : __( "Disabled", "wpep" ) ) ),
        ] + wpep_flag_setting_field_options(),
        'content_before'  => '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_STANDARD . '">',
        'content_after'   => '</div>'
      ],
      'wpep_video_course_overview_page' => [
        'name'    => __( 'Overview Page', "wpep" ),
        'desc'    => '<p>' . __( "Individually chose if this course should have an Index Page / Overview Page", "wpep" ) . '</p>',
        'id'      => 'wpep_video_course_overview_page',
        'type'    => 'select',
        'options' => [
          'default' => sprintf( __( "Course Type Default ( %s )" ), ( wpep_get_setting_flag( 'course-video-allow-index-page' ) ? __( 'Enabled', "wpep" ) : __( "Disabled", "wpep" ) ) ),
        ] + wpep_flag_setting_field_options() ,
        'content_before'  => '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_VIDEO . '">',
        'content_after'   => '</div>'
      ],
    ];

    if( $this->enable_assessments ) {
      $wpep_course_options_fields[ 'wpep_course_assessments_required' ] = [
        'name'    => __( 'Assessments Are', "wpep" ),
        'desc'    => '<p>' . __( "The student is required to complete the section / module assessment in order to continue the course.", "wpep" ) . '</p>',
        'id'      => 'wpep_course_assessments_required',
        'type'    => 'select',
        'options' => [
          1         => __( 'Mandatory', "wpep" ),
          0         => __( 'Optional', "wpep" ),
        ],
      ];
    }

    if( $this->enable_prerequisites_settings ) {
      $wpep_course_options_fields[ WPEP_PREREQUISITE_MODE_POST_META ] = [
        'name'    => __( 'Prerequisites Mode', "wpep" ),
        'desc'    => '<p>' . __( "How the prerequisites logic is handled.", "wpep" ) . '</p>',
        'id'      => WPEP_PREREQUISITE_MODE_POST_META,
        'type'    => 'select',
        'options' => wpep_get_prerequisite_mode_list()
      ];
      $wpep_course_options_fields[ 'wpep_prerequisite' ] = [
        'name'    => __( 'Prerequisites', "wpep" ),
        'desc'    => '<p>' . __( "The student must complete these courses in order to have access to this one.", "wpep" ) . '</p>',
        'id'      => 'wpep_prerequisite',
        'type'    => 'multiple_select',
        'options' => $course_prerequisites
      ];
    }

    if( $this->enable_sidebar_settings ) {
      $wpep_course_options_fields[ 'wpep_sidebar_position' ] = [
        'name'    => __('Sidebar', "wpep"),
        'desc'    =>
          '<p data-wpep-course-type="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . '">' . __( "An Execution Plan will not have a sidebar present if you haven't setup one yourself", "wpep" ) . '</p>' .
          '<p data-wpep-course-type="' . WPEP_COURSE_TYPE_STANDARD . '">' . __( "The Standard Layout will always have a sidebar, we highly recommend disabling it if you know what you're doing.", "wpep" ) . '</p>' .
          '<p data-wpep-course-type="' . WPEP_COURSE_TYPE_VIDEO . '">' . __( "The Video Layout will always have a sidebar while viewing the overview, but disabled during the video library itself.", "wpep" ) . '</p>' .
          '<p class="wpep-not-important">' . sprintf( __( "Normally sidebars are present on the %s" ), ( wpep_is_rtl() ? 'left' : 'right' ) ) . '</p>',
        'id'      => 'wpep_sidebar_position',
        'type'    => 'select',
        'options' => [
          'default' => __( "Default", "wpep" ),
          'right'   => __( 'Right', "wpep" ),
          'bottom'  => __( 'Bottom', "wpep" ),
          'left'    => __( 'Left', "wpep" ),
          'disabled'=> __( 'Disabled', "wpep" )
        ],
        'content_before'  => '<div data-wpep-course-type="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_LEARN_MAP ] ) . '">',
        'content_after'   => '</div>'
      ];
    }
    $wpep_course_options_fields[ 'wpep_read_only' ] = [
      'name'    => __( 'Read Only', "wpep" ),
      'desc'    => '<p>' . __( "This course cannot be completed, all features that track progress are hidden.", "wpep" ) . '</p>',
      'id'      => 'wpep_read_only',
      'type'    => 'select',
      'options' => wpep_flag_setting_field_options(),
    ];
    $wpep_course_options_fields[ 'wpep_show_progress_bar' ] = [
      'name'    => __('Show Progress Bar', "wpep"),
      'desc'    => __('Chose to hide or display the course progress bar specifically for this course.', "wpep"),
      'id'      => 'wpep_show_progress_bar',
      'type'    => 'radio_inline',
      'options' => [
        'show'   => __('Show', "wpep"),
        'hide'   => __('Hide', "wpep")
      ],
      'content_before'  => '<div data-wpep-course-type="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_ONE_OFF ] ) . '">',
      'content_after'   => '</div>'
    ];

    if( $this->enable_presentation_card_settings ) {
      $wpep_course_options_fields[ 'wpep_thumb_image' ] =[
        'name' => __('Thumbnail Image', "wpep"),
        'desc' => __('Set a picture that will show on Course Index Page. ( recommended size : 300px width x 190px height )', "wpep"),
        'id'   => 'wpep_thumb_image',
        'type' => 'image',
      ];

      if( wpep_get_setting_flag( 'index-page-enable-hover-image' ) ) {
        $wpep_course_options_fields[ 'wpep_thumb_image_hover' ] =[
          'name' => __('Thumbnail Image - Hover', "wpep"),
          'desc' => __('Set a picture that will show on Course Index Page on hover.', "wpep"),
          'id'   => 'wpep_thumb_image_hover',
          'type' => 'image',
        ];
      }
    }

    $wpep_course_options_fields[ 'wpep_icon_image' ] =[
      'name'            => __('Icon Image', "wpep"),
      'desc'            => __('Set a picture that will show beside the Course Title. ( recommended size : 100px width x 100px height )', "wpep"),
      'id'              =>  'wpep_icon_image',
      'type'            => 'image',
      'content_before'  => '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . '">',
      'content_after'   => '</div>'
    ];

    return $wpep_course_options_fields;
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( $post_type != WPEP_POST_TYPE_COURSE )
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'shortcode' => __('Shortcode', "wpep"),
    ];

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      $new_columns['statistics'] = __('Statistics', "wpep");

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post;

    if( $post->post_type != WPEP_POST_TYPE_COURSE )
      return;

    if ( $column == 'shortcode' ) {
      $course_type = get_post_meta( $post->ID , 'course_type', true );

      if( in_array( $course_type, [ WPEP_COURSE_TYPE_STANDARD, WPEP_COURSE_TYPE_VIDEO ] ) ) {
        echo sprintf( __( "Not Available for %s Layout", "wpep" ), wpep_alias_to_label( $course_type ) );
        return;
      }

      echo wpep_admin_shortcode_builder(
        'wpep_course',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_course_options'
        ],
        __( "ShortCode", "wpep" ),
        true
      );

      return;
    }

    if( $column == 'statistics' ) {
      echo '<a class="button button-primary" href="' . admin_url('edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-statistics&tab=courses&course_id=' . $post->ID ) . '">' . __( "View Statistics", "wpep" )    . '</a>';

      return;
    }
  }

  public function row_actions($actions, $post) {
    if( $post->post_type != WPEP_POST_TYPE_COURSE )
      return $actions;

    if (current_user_can(WPEP_CAP_EDIT_OTHERS_COURSE ) ) {
      $actions['wpep_duplicate_course'] = '<a href="' . admin_url( 'admin.php?action=wpep_duplicate_course&amp;post=' . $post->ID ) . '" title="Duplicate Course" rel="permalink">Duplicate Course</a>';
    }

    if( get_post_meta( $post->ID , 'course_type', true ) != WPEP_COURSE_TYPE_ONE_OFF ) {
      $actions['wpep_preview'] = wpep_admin_shortcode_builder(
        'wpep_course_preview',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_course_preview_options'
        ],
        __( "Course Preview", "wpep" ),
        false
      );
    }

    return $actions;

  } /* add_duplicate_course_link() */

  public function duplicate_run() {

    global $wpdb;
    if (!(isset($_GET['post']) || isset($_POST['post']) || (isset($_REQUEST['action']) && 'wpep_duplicate_course' == $_REQUEST['action']))) {
      wp_die('No course to duplicate!!!');
    }

    if( !current_user_can(WPEP_CAP_EDIT_COURSE ) )
      wp_die( sprintf( __( "User cannot %s", "wpep" ), WPEP_CAP_EDIT_COURSE ) );

    /*
     * get the original post id
     */
    $post_id = (isset($_GET['post']) ? $_GET['post'] : $_POST['post']);
    /*
     * and all the original post data then
     */
    $post = get_post($post_id);

    /*
     * if you don't want current user to be the new post author,
     * then change next couple of lines to this: $new_post_author = $post->post_author;
     */
    $current_user    = wp_get_current_user();
    $new_post_author = $current_user->ID;

    /*
     * if post data exists, create the post duplicate
     */
    if (isset($post) && $post != null) {

      /*
       * new post data array
       */
      $args = [
        'comment_status' => $post->comment_status,
        'ping_status'    => $post->ping_status,
        'post_author'    => $new_post_author,
        'post_content'   => $post->post_content,
        'post_excerpt'   => $post->post_excerpt,
        'post_name'      => $post->post_name,
        'post_parent'    => $post->post_parent,
        'post_password'  => $post->post_password,
        'post_status'    => 'draft',
        'post_title'     => 'Duplicate Course of ' . $post->post_title,
        'post_type'      => $post->post_type,
        'to_ping'        => $post->to_ping,
        'menu_order'     => $post->menu_order
      ];

      $args = apply_filters( 'wpep_course_duplicate_args', $args );

      /*
       * insert the post by wp_insert_post() function
       */
      $new_post_id = wp_insert_post($args);

      /*
       * get all current post terms ad set them to the new post draft
       */
      $taxonomies = get_object_taxonomies($post->post_type); // returns array of taxonomy names for post type, ex array("category", "post_tag");
      foreach ($taxonomies as $taxonomy) {
        $post_terms = wp_get_object_terms($post_id, $taxonomy, [ 'fields' => 'slugs' ]);
        wp_set_object_terms($new_post_id, $post_terms, $taxonomy, false);
      }

      /*
       * duplicate all post meta
       */
      $post_meta_infos = $wpdb->get_results("SELECT meta_key, meta_value FROM $wpdb->postmeta WHERE post_id=$post_id");
      if (count($post_meta_infos) != 0) {
        $sql_query = "INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) ";
        foreach ($post_meta_infos as $meta_info) {
          $meta_key        = $meta_info->meta_key;
          $meta_value      = addslashes($meta_info->meta_value);
          $sql_query_sel[] = "SELECT $new_post_id, '$meta_key', '$meta_value'";
        }
        $sql_query .= implode(" UNION ALL ", $sql_query_sel);
        $wpdb->query($sql_query);
      }

      update_post_meta( $new_post_id, 'wpep_order', 0 );

      $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_section WHERE post_id = ' . $post_id;

      $course_sections = $wpdb->get_results($sql);

      foreach( $course_sections as $course_section ) {
        $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_section_lesson WHERE section_id = ' . $course_section->id . ' ORDER BY `order` ASC';

        $lessons = $wpdb->get_results($sql);

        unset($course_section->id);
        $course_section->post_id = $new_post_id;

        $wpdb->insert( $wpdb->prefix . 'wpep_section', get_object_vars($course_section) );

        $current_section_id = $wpdb->insert_id;

        foreach( $lessons as $lesson ) {
          unset($lesson->id);

          $lesson->post_id = $new_post_id;
          $lesson->section_id = $current_section_id;

          $wpdb->insert( $wpdb->prefix . 'wpep_section_lesson', get_object_vars($lesson) );
        }
      }

      /*
       * finally, redirect to the edit post screen for the new draft
       */
      wp_redirect(admin_url('post.php?action=edit&post=' . $new_post_id));
      exit;
    } else {
      wp_die('Post creation failed, could not find original post: ' . $post_id);
    }

  }

}
