<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

class PlatformHealth {

  public $tabs;
  private $_error_storage_transient_name = 'wpep_platform_health_error_detection';

  public function setup() {
    add_action( 'admin_menu', [ $this, '_register_menu' ], 14 );
    add_action( 'init', [ $this, '_init' ] );
    add_action( 'activated_plugin', [ $this, '_any_plugin_activation_deactivation' ], 10 );
    add_action( 'deactivated_plugin', [ $this, '_any_plugin_activation_deactivation' ], 10 );
  }

  public function _register_menu() {
    add_submenu_page(
      'edit.php?post_type=' . WPEP_POST_TYPE_COURSE,
      __("WPEP Platform Health", 'wpep'),
      __( "Platform Health", 'wpep'),
      WPEP_CAP_MANAGE_SETTINGS,
      "wpep-platform-health",
      [ $this, "display" ]
    );
  }

  public function _init() {
    if( !is_admin() )
      return;

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;

    if( $this->_get_system_services_error_count() === 0 )
      return;

    if( isset( $_GET['post_type'] ) && isset( $_GET['page'] ) )
      if( $_GET['post_type'] == WPEP_POST_TYPE_COURSE && $_GET['page'] == 'wpep-platform-health' )
        return;

    add_action('admin_notices', [ $this, 'admin_notices' ] );
  }

  public function display() {
    echo '<div class="wrap">';
    echo '<h2>' . __( "WPEP - Platform Health", 'wpep') . '</h2>';
    echo '<hr/>';

    $current_platform_health_page = $this->get_pages( $this->get_current_tab() );

    $this->show_tabs( $this->get_current_tab() );

    $current_platform_health_page->display();

    echo '</div>';
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="nav-tab-wrapper">';
    foreach( $this->get_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' nav-tab-active' : '';
      echo "<a class='nav-tab$class' href='" . wpep_admin_url_platform_health( [ 'tab' => $tab ] ). "'>" . $setting_page->get_name() . "</a>";

    }
    echo '</h2>';
  }

  public function get_current_tab() {
    $statistics_pages = $this->get_pages();

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $statistics_pages ) );

    if( !isset( $statistics_pages[ $current_tab ] ) )
      $current_tab = key( $statistics_pages );

    return $current_tab;
  }

  public function get_pages( $index = false ) {
    if( !is_array( $this->tabs ) ) {
      $this->tabs = [];

      foreach( [
                  new PlatformHealth\Dashboard(),
                  new PlatformHealth\Export(),
                  new PlatformHealth\CleanupUtilities(),
                  new PlatformHealth\Tools(),
               ] as $current_instance )
        $this->tabs[ $current_instance->get_alias() ] = $current_instance;

      $this->tabs = apply_filters( 'wpep_administration_platform_health_pages', $this->tabs );
    }

    return ( $index == false ? $this->tabs : $this->tabs[$index] );
  }

  public function _any_plugin_activation_deactivation() {
    $this->delete_error_transient_storage();
  }

  public function delete_error_transient_storage() {
    delete_transient( $this->_error_storage_transient_name );
  }

  public function admin_notices() {
    echo '<div class="error">';
    echo    '<p>';
    echo      __( 'Warning! There are WPEP Platform Health issues that require your attention.', "wpep" ) .
             ' <strong>' . sprintf( __( '%sPlease click here%s and click on the items highlighted in red to resolve them now.', "wpep" ), '<a href="' . wpep_admin_url_platform_health() . '">', '</a>' ) . '</strong>';
    echo    '</p>';
    echo '</div>';
  }

  private function _get_system_services_error_count() {
    $error_count = get_transient( $this->_error_storage_transient_name );

    if( $error_count !== false )
      return intval( $error_count );

    $error_count = 0;

    foreach( WPEP_Controller::instance()->platformHealth->get_service_status_report() as $service_status_information ) {
      if( !isset( $service_status_information['status'] ) )
        continue;

      if( $service_status_information['status'] !== 'danger'
          && $service_status_information['status'] !== 'error' )
        continue;

      $error_count++;
    }

    set_transient( $this->_error_storage_transient_name, $error_count, 3600 );

    return $error_count;
  }

}
