<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Statistics {

  /**
   * @var array
   */
  protected $_statisticsPages;

  public function setup() {
    add_action( 'admin_menu', [ $this, '_register_menu' ] );
    add_action( 'init', [ $this, '_init' ] );
  }

  public function filter_by_user_role() {
    $user_filters[] = [
      'user_role' => false
    ];

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      $user_filters = get_user_meta( get_current_user_id(), 'wpep_statistics_filter_settings', true );

    $user_filters = apply_filters( 'wpep_administration_statistics_user_filters', $user_filters );

    return isset(  $user_filters['user_role' ] ) && $user_filters['user_role'] !== '' ? $user_filters['user_role'] : false;
  }

  private function _get_cap() {
    return apply_filters( 'wpep_administration_statistics_user_cap', WPEP_CAP_MANAGE_STATISTICS );
  }

  public function _init() {
    if( !is_admin() )
      return;

    if( !current_user_can( $this->_get_cap() ) )
      return;

    if( !isset( $_GET['post_type'] ) || !isset( $_GET['page'] ) )
      return;

    if( $_GET['post_type'] != WPEP_POST_TYPE_COURSE || $_GET['page'] != 'wpep-statistics' )
      return;

    if( isset( $_POST['wpep_user_role_filter'] ) && current_user_can( WPEP_CAP_MANAGE_STATISTICS ) ) {
      update_user_meta( get_current_user_id(), 'wpep_statistics_filter_settings', [
        'user_role' => sanitize_text_field( $_POST['wpep_user_role_filter'] )
      ] );
    }

    if( !isset( $_POST['wpep_download_statistics'] ) )
      return;

    $current_statistics_page = $this->get_statistics_pages( $this->get_current_tab() );

    if( method_exists( $current_statistics_page, 'download' ) )
      $current_statistics_page->download( [
        'user_role' => $this->filter_by_user_role()
      ] );
  }

  public function _register_menu() {
    add_submenu_page(
      'edit.php?post_type=' . WPEP_POST_TYPE_COURSE,
      __("WPEP Statistics", 'wpep'),
      __( "Statistics", 'wpep'),
      $this->_get_cap(),
      "wpep-statistics",
      [ $this, "display" ]
    );
  }

  public function display() {
    echo '<div class="wrap">';

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      echo $this->_get_user_role_filter_dropdown();
    else
      echo '<h2>' . __( "WPEP Statistics", 'wpep') . '</h2>';

    echo '<hr/>';

    $current_statistics_page = $this->get_statistics_pages( $this->get_current_tab() );

    $this->show_tabs( $this->get_current_tab() );

    $current_statistics_page->display( [
      'user_role' => $this->filter_by_user_role()
    ] );

    echo '</div>';
  }

  private function _get_user_role_filter_dropdown() {
    $role_filter = '';

    $editable_roles = array_reverse( get_editable_roles() );

    $role_filter .= '<form method="POST" id="wpep_user_role_filter_container">';
    $role_filter .=   '<h2>' . __( "WPEP Statistics", 'wpep') . '</h2>';

    $role_filter .=   '<select id="wpep_user_role_filter" name="wpep_user_role_filter">';

    $role_filter .= "\n\t<option value=''>" . __( "All Roles", "wpep" ) . "</option>";

    foreach ( $editable_roles as $role => $details ) {
      $name = translate_user_role($details['name'] );
      // preselect specified role
      if ( $this->filter_by_user_role() == $role ) {
        $role_filter .= "\n\t<option selected='selected' value='" . esc_attr( $role ) . "'>$name</option>";
      } else {
        $role_filter .= "\n\t<option value='" . esc_attr( $role ) . "'>$name</option>";
      }
    }

    $role_filter .=   '</select>';
    $role_filter .=   '<div class="clear"></div>';
    $role_filter .= '</form>';

    return $role_filter;
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="nav-tab-wrapper">';
    foreach( $this->get_statistics_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' nav-tab-active' : '';
      echo "<a class='nav-tab$class' href='" . wpep_admin_url_statistics( [ 'tab' => $tab ] ). "'>" . $setting_page->get_name() . "</a>";

    }
    echo '</h2>';
  }

  public function get_current_tab() {
    $statistics_pages = $this->get_statistics_pages();

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $statistics_pages ) );

    if( !isset( $statistics_pages[ $current_tab ] ) )
      $current_tab = key( $statistics_pages );

    return $current_tab;
  }

  public function get_statistics_pages( $index = false ) {
    if( !is_array( $this->_statisticsPages ) ) {
      $this->_statisticsPages = [];

      $objectInstances = [];

      if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
        $objectInstances[] = new Statistics\Overview();

      $objectInstances[] = new Statistics\Courses();

      if( wpep_assessments_enabled() )
        $objectInstances[] = new Statistics\Assessments();

      if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) || current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
        $objectInstances[] = new Statistics\Users();
      
      if( wpep_get_setting_flag( 'enable-course-timing-option' )
          && ( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) || current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ))
        $objectInstances[] = new Statistics\UserNotifications();

      if( wpep_get_setting_flag( 'enable-course-timing-option' )
        && ( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) || current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ))
        $objectInstances[] = new Statistics\GAReport();

      foreach( $objectInstances as $current_instance )
        $this->_statisticsPages[ $current_instance->get_alias() ] = $current_instance;

      $this->_statisticsPages = apply_filters( 'wpep_administration_statistics_pages', $this->_statisticsPages );
    }

    return ( $index == false ? $this->_statisticsPages : $this->_statisticsPages[$index] );
  }

}
