<?php

namespace WPEP\Container;

trait Migration {

  /**
   * @var Migration\Admin
   */
  public $admin;

  public $has_admin                    = false;
  public $namespace                    = false;
  public $plugin_file_path             = false;
  public $plugin_migration_folder_path = false;
  public $option_name                  = false;
  public $option_progress_name         = false;
  public $current_version              = false;
  public $current_version_available    = false;
  public $migration_list               = [];

  public function get_name() {
    return "WPEP";
  }

  public function get_menu_name() {
    return "Migration";
  }

  public function trait_setup( $variables ) {
    $this->_setup_variables( $variables );

    register_activation_hook( $this->plugin_file_path, [ $this, '_plugin_activation_hook' ] );

    if( method_exists( $this, 'wpmu_add_new_blog' ) )
      add_action( 'wpmu_new_blog', [ $this, 'wpmu_add_new_blog' ] );

    if( method_exists( $this, 'wpmu_drop_tables' ) )
      add_filter( 'wpmu_drop_tables', [ $this, 'wpmu_drop_tables' ] );

    if( $this->has_admin )
      $this->admin = new Migration\Admin( $this );
  }

  private function _setup_variables( $variables = [] ) {
    if( is_array( $variables ) && !empty( $variables ) ) {
      foreach( $variables as $variable_name => $variable_value )
        $this->$variable_name = $variable_value;
    }

    $this->current_version = get_option( $this->option_name, '0.0.0' );

    if( $this->namespace === false ) {
      $this->namespace = basename( $this->plugin_file_path );
      $this->namespace = str_replace( '.php', '',  $this->namespace );
    }
  }

  public function get_current_migration_version() {
    foreach( $this->migration_list as $migration_version => $migration_file )
      if( version_compare( $migration_version, $this->current_version, ">")  )
        return $migration_version;

    return false;
  }

  public function is_migration_required() {
    return version_compare( $this->current_version_available, $this->current_version, ">");
  }

  public function migrate_to_version( $version_number ) {
    $last_migration = get_option( $this->option_progress_name, 0 );

    if( time() - $last_migration < 30 )
      return "<p>" . sprintf( __( "Migration currently marked in progress, it started %s seconds.", "wpep" ), ( $last_migration == 0 ? 'too many' : time() - $last_migration ) ). "</p>" .
             '<p><a class="button button-primary wpep-migration-continue">' . __( "Retry", "wpep" ) . '</a></p>';

    if( !version_compare( $version_number, $this->current_version, ">") && $this->current_version != 0 )
      return "<p>" . __( "This version has already been installed, skipping", "wpep" ) . "</p>";

    update_option( $this->option_progress_name, time() );

    $response = $this->_migrate_to_version( $version_number );

    delete_option( $this->option_progress_name );

    return $response;
  }

  private function _migrate_to_version( $migrate_to_version ) {
    $migrate_to_version_response = '';

    $migration_okay = true;

    foreach( $this->migration_list as $migration_version => $migration_file ) {
      if ( version_compare( $this->current_version, $migration_version ) != -1 )
        continue;

      if( !file_exists( $this->plugin_migration_folder_path . $migration_file ) ){
        $migrate_to_version_response .= '<p class="wpep-notification wpep-notification-danger">' . sprintf( __( "File not found %s.", "wpep" ), $this->plugin_migration_folder_path . $migration_file ). '</p>';
        $migrate_to_version_response .= '<p><a class="button button-primary wpep-migration-continue">' . __( "Retry", "wpep" ) . '</a></p>';
        $migration_okay = false;
        break;
      }

      if ( version_compare( $migrate_to_version, $migration_version, "<") )
        break;

      ob_start();

      require_once( $this->plugin_migration_folder_path . $migration_file );

      $migrate_to_version_response = ob_get_contents() . $migrate_to_version_response;

      ob_end_clean();
    }

    if( $migration_okay ) {
      $this->current_version = $migrate_to_version;

      update_option( $this->option_name, $migrate_to_version );
    }

    return $migrate_to_version_response;
  }

  public function _plugin_activation_hook( $network_wide ) {
    if (function_exists( 'is_multisite' ) && is_multisite() && $network_wide ) {
      $this->_plugin_activation_hook_network();
      return;
    }

    if( !$this->handle_migration_in_background() ) {
      $this->_maybe_after_activation_methods();
      return;
    }

    $this->migrate_to_version( $this->current_version_available );

    $this->_maybe_after_activation_methods( true );
  }

  public function _plugin_activation_hook_network() {
    global $wpdb;

    $base_blog_id = $wpdb->blogid;
    $blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

    foreach ( $blog_ids as $blog_id ) {
      switch_to_blog($blog_id);

      if( !$this->handle_migration_in_background() ) {
        $this->_maybe_after_activation_methods( false );
        continue;
      }

      $this->migrate_to_version( $this->current_version_available );

      if( method_exists( $this,'_after_background_migration' ) )
        $this->_maybe_after_activation_methods( true );
    }

    switch_to_blog( $base_blog_id );

    return NULL;
  }

  private function _maybe_after_activation_methods( $migration_done = false ) {
    if( $migration_done && method_exists( $this,'_after_background_migration' ) )
      $this->_after_background_migration();

    if( method_exists( $this,'_after_activation' ) )
      $this->_after_activation();
  }

  public function wpmu_add_new_blog( $blog_id ) {
    if ( !is_plugin_active_for_network( WPEP_PLUGIN_IDENTIFIER ) )
      return;

    switch_to_blog( $blog_id );

    $this->_setup_variables();
    restore_current_blog();
  }

  public function wpmu_drop_tables( $tables ) {
    return $tables;
  }

  public function handle_migration_in_background() {
    if( $this->current_version != '0.0.0' )
      return false;

    return true;
  }

}