<?php
namespace WPEP\Coordinator\Ajax;

class Auth {

  public function __construct() {
    add_action('wp_ajax_wpep_auth_request', [ $this, 'auth_request_logged' ] );
    add_action('wp_ajax_nopriv_wpep_auth_request', [ $this, 'auth_request' ] );
  }

  public function auth_request() {
    if( !isset( $_POST['auth_information'] ) || !isset( $_POST['auth_type'] ) )
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => __( "Invalid Request, the page will reload", "wpep" ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ] ) );

    $auth_type        = $_POST['auth_type'];
    $auth_information = $_POST['auth_information'];

    $allow_registration = ( wpep_get_setting_flag( 'user-auth-enable-registration' ) || wpep_get_setting_flag( 'enable-post-member-magnet-option' ) );
    $allow_registration = apply_filters( 'wpep_auth_allow_registration', $allow_registration );

    if( !isset( $auth_information['wpep_auth_nonce'] ) || wp_verify_nonce( $auth_information['wpep_auth_nonce'], 'wpep_auth_nonce' ) == false )
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-auth-notification-page-expired' ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ] ) );

    if( $auth_type == 'login' ) {
      if( $auth_information['login_identifier'] == '' || $auth_information['password'] == '' )
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-login-details-required' ),
          'notification_class'  => 'wpep-notification'
        ], $auth_information, $auth_type ) );

      $user = wp_signon( [
        'user_login'    => $auth_information['login_identifier'],
        'user_password' => $auth_information['password'],
        'remember'      => true
      ], false );

      if ( is_wp_error( $user ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => ( isset( $user->errors['incorrect_password'] )
            ? ( wpep_get_setting_translated( "user-auth-notification-password-incorrect" ) . ' <span class="reset-password-text" data-field-type-switch="reset_password">' . wpep_get_setting_translated( 'user-auth-open-tab-reset-password' ) . '</span>' )
            : $user->get_error_message()
          ),
          'notification_class'  => 'wpep-notification wpep-notification-warning'
        ], $auth_information, $auth_type ) );
      } else {
        exit ( $this->_auth_request_response( [
          'status'              => 'ok',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-logged-in' ),
          'notification_class'  => 'wpep-notification wpep-notification-success',
          'refresh'             => 1,
          'refresh_delay'       => 1000
        ], $auth_information, $auth_type ) );
      }

    } else if( $auth_type == 'register' && $allow_registration ) {
      $user_id = wp_create_user( $auth_information['username'], $auth_information['password'], $auth_information['email_address'] );

      if( is_wp_error( $user_id ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => $user_id->get_error_message(),
          'notification_class'  => 'wpep-notification wpep-notification-warning'
        ], $auth_information, $auth_type ) );
      } else {
        $first_name = sanitize_text_field( $auth_information['first_name'] );
        $last_name  = sanitize_text_field( $auth_information['last_name'] );

        wp_update_user( [
          'ID'           => $user_id,
          'first_name'   => $first_name,
          'last_name'    => $last_name,
          'display_name' => ( $first_name == '' && $last_name == '' ? $auth_information['username'] : trim( $first_name . ' ' . $last_name ) )
        ] );

        $user = get_user_by( 'id', $user_id );

        wp_set_current_user( $user_id, $user->user_login );
        wp_set_auth_cookie( $user_id );
        do_action( 'wp_login', $user->user_login );

        exit ( $this->_auth_request_response( [
          'status'              => 'ok',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-account-registered' ),
          'notification_class'  => 'wpep-notification wpep-notification-success',
          'refresh'             => 1,
          'refresh_delay'       => 2000
        ], $auth_information, $auth_type ) );
      }
    } else if( $auth_type == 'reset_password' ) {
      $response = wpep_retrieve_password( $auth_information['login_identifier'] );

      if( is_wp_error( $response ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => $response->get_error_message(),
          'notification_class'  => 'wpep-notification wpep-notification-warning'
        ], $auth_information, $auth_type ) );
      } else {
        exit ( json_encode( [
          'status'              => 'ok',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-password-reset' ),
          'notification_class'  => 'wpep-notification wpep-notification-success',
          'refresh'             => 1,
          'refresh_delay'       => 5000
        ] ) );
      }
    }

    exit ( $this->_auth_request_response( [
      'status'              => 'error',
      'notification'        => __( "An unexpected error has just occured, please try again in a few moments.", "wpep" ),
      'notification_class'  => 'wpep-notification wpep-notification-danger',
      'refresh'             => 1
    ], $auth_information, $auth_type ) );
  }

  private function _auth_request_response( $response, $auth_information, $auth_type ) {
    if( !isset( $response['status'] ) || $response['status'] != 'ok' )
      return json_encode( $response );

    if( $auth_type == 'register' && isset( $auth_information[ 'wpep_post_id'] ) ) {
      $post_id = intval( $auth_information[ 'wpep_post_id'] );

      update_user_meta( get_current_user_id(), 'wpep_registration_post_id', $post_id );

      if( wpep_post_has_member_magnet_option( $post_id ) ) {
        $custom_redirect = trim( get_post_meta( $post_id, 'wpep_member_magnet_option_registration_redirect_url', true ) );

        if( $custom_redirect !== '' ) {
          wpep_user_course_started_event( $post_id, get_current_user_id() );

          $response = [
            'status'       => 'ok',
            'redirect'     => $custom_redirect,
            'redirect_top' => 1
          ];
        }
      }
    }

    return json_encode( apply_filters( "wpep_auth_request_response_json", $response, $auth_information, $auth_type ) );
  }

  public function auth_request_logged() {
    exit ( json_encode( [
      'notification'        => wpep_get_setting_translated( 'user-auth-notification-already-logged-in' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'refresh'             => 1,
      'refresh_delay'       => 2000
    ] ) );
  }

}