<?php

namespace WPEP\Coordinator\Content\Single;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;
use WPEP\Entity\Course as WPEP_Entity_Course;

class Video extends Skeleton {

  public $current_item = 'course';
  public $current_item_id = 0;

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = true;

  /**
   * Standard constructor.
   */
  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->course_has_index_page             = wpep_get_setting_flag( 'course-video-allow-index-page' );
    $this->course_has_redirect_on_completion = wpep_get_setting_flag( 'course-video-redirect-on-completion' );

    $course_has_index_page = get_post_meta( $this->course_id, 'wpep_video_course_overview_page', true );

    if( $course_has_index_page != 'default' )
      $this->course_has_index_page = $course_has_index_page;
  }

  public function get_alias() {
    return "video";
  }

  public function init() {
    do_action( 'wpep_course_single_content_video_coordinator_before_init', $this );

    if( isset( $_GET[$this->lesson_slug] ) ) {
      $this->current_item     = $this->lesson_slug;
      $this->current_item_id  = intval( $_GET[$this->lesson_slug] );
    }

    $this->_init_item_based_course();

    do_action( 'wpep_course_single_content_video_coordinator_after_init', $this );

    if( $this->current_item == $this->lesson_slug ) {
      add_filter( 'wpep_single_content_sidebar_active', '__return_false', 50 );
    } else {
      add_filter( 'wpep_single_content_sidebar_active', '__return_true', 50 );
      add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    }
  }

  public function content() {
    if( $this->current_item == $this->lesson_slug )
      return $this->get_content_lesson( $this->current_item_id );

    return $this->get_content();
  }

  public function get_content() {
    $course_sections = WPEP_Controller::instance()->userActivity->get_course_sections( $this->course_id, get_current_user_id() );

    $args = [
      'course_id'         => $this->course_id,
      'course_read_only'  => $this->read_only,
      'course_sections'   => $course_sections,
      'course_title'      => get_the_title( $this->course_id ),
      'course_content'    => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
      'course_status'     => $this->_get_course_status( $course_sections )
    ];

    $args = apply_filters( 'wpep_course_single_content_video_args', $args );

    return apply_filters(
      'wpep_single_content_video_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/video-content.php', $args ),
      $args
    );
  }

  private function _get_course_status( $course_sections ) {
    $completed_counter = 0;

    foreach( $course_sections as $course_section )
      if( $course_section['is_completed'] )
        $completed_counter++;

    if( $completed_counter == count( $course_sections ) )
      return 'completed';

    if( $completed_counter != 0 )
      return 'progress';

    foreach( $course_sections as $course_section )
      foreach( $course_section['lessons'] as $course_section_lesson )
        if( $course_section_lesson['is_completed'] == 1 )
          return 'progress';

    return 'pending';
  }

  public function get_content_lesson( $lesson_id ) {
    $course_lesson = WPEP_Controller::instance()->userActivity->get_course_lesson( $lesson_id, get_current_user_id(), true, null );

    if( $course_lesson == null )
      return '';

    $course_section_information = WPEP_Controller::instance()->userActivity->get_section_map( $course_lesson->section_id, get_current_user_id() );

    $lesson_options = ( is_string( $course_lesson->options ) && $course_lesson->options != '' ? json_decode( $course_lesson->options ) : $course_lesson->options );
    $lesson_content = wpautop( $course_lesson->content );

    if( isset( $lesson_options->downloads_list ) && !empty( $lesson_options->downloads_list ) ) {
      $lesson_content .= WPEP_Controller::instance()->template->get_template( 'utilities/downloads-list.php', [
        'downloads_list'  => $lesson_options->downloads_list,
        'title'           => ''
      ] );
    }

    $args = [
      'lesson_list'         => $course_section_information->lessons,
      'course_read_only'    => $this->read_only,
      'course_id'           => $this->course_id,
      'course_name'         => get_the_title( $this->course_id ),
      'lesson_id'           => $course_lesson->id,
      'lesson_title'        => $course_lesson->title,
      'lesson_content'      => $lesson_content,
      'lesson_options'      => $lesson_options,
      'lesson_is_completed'       => $course_lesson->is_completed,
      'next_lesson_id'            => $course_lesson->next_lesson_id,
      'next_lesson_title'         => $course_lesson->next_lesson_title,
      'previous_lesson_id'        => $course_lesson->previous_lesson_id,
      'previous_lesson_title'     => $course_lesson->previous_lesson_title,
      'has_index_page'      => $this->course_has_index_page
    ];

    $args = apply_filters( 'wpep_course_single_content_video_lesson_args', $args );

    return apply_filters(
      'wpep_single_content_video_lesson_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/video-lesson-content.php', $args ),
      $args
    );
  }

  public function display_sidebar_content() {
    echo '<div class="wpep-widget wpep-course-widget-primary">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<div class="video-course-thumbnail-widget">';
    echo        '<img src="' . wpep_course_thumbnail_url( $this->course_id ) . '"/>';
    echo        '<h2>' . get_the_title( $this->course_id ) . '</h2>';
    echo      '</div>';
    echo    '</div>';
    echo '</div>';

    if( wpep_course_has_progress_bar( $this->course_id ) ) {
      echo '<div class="wpep-widget">';
      echo    '<div class="wpep-widget-wrapper">';
      echo      '<div class="video-course-progress-widget">';
      echo        '<h2>' . wpep_get_setting_translated( 'course-video-course-progress' ) . '</h2>';

      echo         wpep_progress_bar( WPEP_Controller::instance()->userActivity->course_progress_percentage( $this->course_id, get_current_user_id() )  );
      echo      '</div>';
      echo    '</div>';
      echo '</div>';
    }

  }

}