<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class TemplateStyle {

  public $layout_option_list = [ 'layout-max-width', 'layout-background-color', 'layout-padding', 'layout-box-shadow' ];
  public $element_map = [];
  public $default_theme   = 'renewal';
  public $theme           = 'legacy';
  public $theme_list      = [];

  public $custom_style_delivered = false;
  /**
   * @var array - options that are not provided will be taken from the settings.
   */
  public $request_layout_options = [];

  public function __construct() {
    $this->theme = WPEP_Settings::instance()->get_option( 'wpep-theme' );
  }

  public function setup() {
    $this->_setup_themes();

    add_action( 'init', [ $this, '_load_element_map' ], 20 ); // Needs to hook up later, as theme may be changed.
    add_action( 'wpep_before_main_content', [ $this, 'before_main_content' ] );
  }

  public function _load_element_map() {
    if( !isset( $this->theme_list [ $this->theme ] ) )
      $this->theme = $this->default_theme;

    $primary_color   = trim( WPEP_Settings::instance()->get_option( 'wpep-custom-color', '' ) );

    $this->element_map = [
      'color_primary'                      => $primary_color,
      'color_secondary'                    => trim( WPEP_Settings::instance()->get_option( 'wpep-custom-color-secondary', '' ) ),
      'font_size'                          => WPEP_Settings::instance()->get_option( 'wpep-custom-font-size', '' ),
      'font_family'                        => WPEP_Settings::instance()->get_option( 'wpep-custom-font-family', '' ),
      'index_page_button_color'            => WPEP_Settings::instance()->get_option( 'index-page-start-text-color', '' ),
      'index_page_button_background_color' => WPEP_Settings::instance()->get_option( 'index-page-start-text-color-background', '' ),
      'index_page_ribbon_color'            => WPEP_Settings::instance()->get_option( 'index-page-ribbon-text-color', '' ),
      'index_page_ribbon_background_color' => WPEP_Settings::instance()->get_option( 'index-page-ribbon-text-color-background', '' )
    ];


    if( !isset( $this->theme_list [ $this->theme ]['defaults'] ) )
      return;

    foreach( $this->theme_list [ $this->theme ]['defaults'] as $key => $value ) {
      if( !isset( $this->element_map[$key] ) ) {
        $this->element_map[ $key ] = $value;

        continue;
      }

      if( trim( $this->element_map[$key] ) == '' )
        $this->element_map[$key] = $value;
    }
  }

  /**
   * Compatibility if Custom CSS was not loaded trough a File
   */
  public function before_main_content() {
    if( $this->custom_style_delivered && !is_admin() )
      return;

    echo '<style type="text/css">';
    echo  $this->get_custom_style();
    echo '</style>';

    $this->custom_style_delivered = true;
  }

  public function get_wpep_wrapper_style() {
    $layout_options = $this->get_request_layout_options();

    $ret = '';
    $ret .= ( $layout_options['layout-max-width']        != 0           ? 'margin: 0 auto; max-width : ' . $layout_options['layout-max-width'] . 'px;' : '' );
    $ret .= ( $layout_options['layout-background-color'] != ''          ? 'background-color : ' . $layout_options['layout-background-color'] . ';' : '' );
    $ret .= ( $layout_options['layout-padding']          != 'default'   ? (
      'padding : ' . $layout_options['layout-padding'] . ( !is_numeric( $layout_options['layout-padding'] ) || $layout_options['layout-padding'] == 0 ? '' : 'px' ) . ';'
    ) : '' );
    $ret .= ( $layout_options['layout-box-shadow']       != 'default'   ? 'box-shadow : ' . $layout_options['layout-box-shadow'] . ';' : '' );

    return $ret;
  }

  public function get_request_layout_options() {
    $layout_options = $this->request_layout_options + WPEP_Settings::instance()->get_option( $this->layout_option_list );

    $layout_options = apply_filters( 'wpep_layout_options_css_args', $layout_options );

    return $layout_options;
  }

  public function match_request_layout_options( $options ) {
    if( !is_array( $options ) )
      return;

    foreach( $options as $option_key => $option_value )
      if( in_array( $option_key, $this->layout_option_list ) )
        $this->set_request_layout_option( $option_key, $option_value );
  }

  public function set_request_layout_option( $option_name, $option_value ) {
    $this->request_layout_options[ $option_name ] = $option_value;
  }

  public function get_custom_style( $wpep_theme = false ) {
    if( $wpep_theme !== false && $wpep_theme != $this->theme ) {
      $prev_theme = $this->theme;

      $this->theme = $wpep_theme;
      $this->_load_element_map();
    }

    $content = '';

    $content .= $this->_get_platform_custom_css();
    $content .= WPEP_Settings::instance()->get_option('wpep-custom-css');

    if( isset( $prev_theme ) ) {
      $this->theme = $prev_theme;

      $this->_load_element_map();
    }

    return $content;
  }
  
  private function _setup_themes() {
    $this->theme_list['legacy'] = [
      'name'                 => 'Legacy',
      'alias'                => 'legacy',
      'style_file'           => WPEP_BASE_URL_PATH . 'assets/front-themes/legacy/style.css',
      'style_rtl_file'       => WPEP_BASE_URL_PATH . 'assets/front-themes/legacy/rtl-support.css',
      'style_variation_file' => WPEP_BASE_URL_PATH . 'assets/front-themes/legacy/color-variation.css',
      /** 'style_font_file' => '', **/
      'defaults'             => [
        'color_primary'   => '#BDD440',
        'color_secondary' => '#607106',
        'font_size'       => '14',
        'font_family'     => 'Open Sans',
        'index_page_ribbon_color'             => '#ffffff',
        'index_page_ribbon_background_color'  => '#168bea'
      ]
    ];
    $this->theme_list['renewal'] = [
      'name'           => 'Renewal',
      'alias'          => 'renewal',
      'style_file'     => WPEP_BASE_URL_PATH . 'assets/front-themes/renewal/style.css',
      'style_rtl_file' => WPEP_BASE_URL_PATH . 'assets/front-themes/renewal/rtl-support.css',
      'style_variation_file' => WPEP_BASE_URL_PATH . 'assets/front-themes/renewal/color-variation.css',
      'defaults'             => [
        'color_primary'   => '#168bea',
        'color_secondary' => '#25c348',
        'font_size'       => '14',
        'font_family'     => 'Noto Sans',
        'index_page_ribbon_color'             => '#ffffff',
        'index_page_ribbon_background_color'  => '#168bea'
      ]
    ];

    $this->theme_list = apply_filters( 'wpep_available_theme_list', $this->theme_list );
  }

  public function get_theme_list() {
    return $this->theme_list;
  }

  public function get_current_theme() {
    return  $this->theme_list[ $this->theme ];
  }

  private function _get_platform_custom_css() {
    if( empty( $this->element_map ) )
      $this->_load_element_map();

    $content = '';

    if( !in_array( $this->element_map['font_family'], wpep_get_native_font_list() ) )
      $content .= '@import url(https://fonts.googleapis.com/css?family=' . str_replace( ' ', '+', $this->element_map['font_family'] ) . ':300,400,500,600,700,800);' . "\n";

    if( isset( $this->get_current_theme()['style_variation_file'] ) )
      $content .= $this->_get_css_file_content( $this->get_current_theme()['style_variation_file'] );

    $style_file_path = ( isset( $this->get_current_theme()['style_font_file'] ) ? $this->get_current_theme()['style_font_file'] : false );

    if( is_string( $style_file_path ) || strlen( $style_file_path ) <= 10 )
      $style_file_path = WPEP_BASE_URL_PATH . 'assets/front-themes/_common/font.css';

    $content .= $this->_get_css_file_content( $style_file_path );

    foreach( $this->element_map as $replace_identifier => $replace_value ) {
      if( $replace_identifier == 'font_family' ) {
        $replace_value      = ( $replace_value === 'inherit' ? $replace_value : $replace_value . ", sans-serif" );
      } else if( $replace_identifier == 'font_size' ) {
        if( $replace_value == WPEP_BASE_FONT_SIZE ) // Base Font-Size
          continue;

        $replace = [];

        for( $i = - WPEP_BASE_FONT_SIZE; $i <= 70; $i++ )
          $replace[ 'font-size:' . ( WPEP_BASE_FONT_SIZE + $i ) . "px" ] = 'font-size:' . ( $replace_value + $i > 0 ? $replace_value + $i . "px" : 0 );

        $content = strtr( $content, $replace );

        continue;
      }

      $content = str_replace( '#' . $replace_identifier, $replace_value, $content );
    }

    return apply_filters('wpep_platform_specific_css', $content );
  }

  private function _get_css_file_content( $file_path ) {
    $style_variation = '';

    try {
      $style_variation = @file_get_contents( str_replace( get_home_url(), ABSPATH . '/', $file_path ) );

      if( $style_variation == '' )
        $style_variation = @file_get_contents( $file_path );
    } catch ( \Exception $exception ) {
      $style_variation = '';
    }

    if ( $style_variation != '' )
      return $style_variation . "\n";

    if( strpos($file_path, 'http') !== 0 )
      return $style_variation . '\n';

    try {
      $response = wp_remote_get($file_path);

      if (!is_wp_error($response)) {
        $response = wp_remote_retrieve_body($response);

        if (is_string($response) && $response != '')
          $style_variation = $response;
      }
    } catch ( \Exception $exception ) {

    }

    return $style_variation . "\n";
  }

}
