<?php

/**
 * @return int
 */
function wpep_get_auth_status_level() {
  $auth_status_level = wpep_get_setting_flag( 'user-auth-status-level' );

  $auth_status_level = apply_filters( 'wpep_auth_status_level', $auth_status_level );

  return $auth_status_level;
}

/**
 * @param null $index
 * @return array|string
 */
function wpep_get_auth_status_level_options( $index = null ) {
  $options = [
    0 => 'Users are not required to login in order to access WPEP Content',
    1 => 'Users are required to login in order to view Courses and Videos',
    2 => 'Users are required to login in order to view the Index Page, Category Pages, Courses, Videos',
    3 => 'Users are required to login in order to view the Index Page, Category Pages, Courses, Course ShortCode Embeds, Index Page Embeds, Videos'
  ];

  return ( $index !== NULL ? $options[ $index ] : $options );
}

/**
 * @return array
 */
function wpep_get_auth_status_page_restrictions() {
  $status_level      = intval( wpep_get_auth_status_level() );
  $page_restrictions = [];

  if( $status_level >= 1 )
    $page_restrictions = array_merge( $page_restrictions, [ 'content-course-single.php', 'content-video-single.php', 'content-assessment-single.php' ] );

  if( $status_level >= 2 )
    $page_restrictions = array_merge( $page_restrictions, [ 'primary.php', 'primary-category.php' ] );

  if( $status_level >= 3 )
    $page_restrictions = array_merge( $page_restrictions, [ 'course-shortcode', 'index-page-shortcode', 'course-grid-shortcode' ] );

  return $page_restrictions;
}

/**
 * Same Function like wp-login, we'll usually try to sink them and modify where needed
 * @return bool|WP_Error
 */
function wpep_retrieve_password( $user_login ) {
  $errors = new WP_Error();

  if ( strpos( $user_login, '@' ) ) {
    $user_data = get_user_by( 'email', trim( wp_unslash( $user_login ) ) );
    if ( empty( $user_data ) )
      $errors->add( 'invalid_email', __('There is no user registered with that email address.', "wpep" ) );
  } else {
    $login = trim( $user_login );
    $user_data = get_user_by('login', $login);
  }

  /**
   * Fires before errors are returned from a password reset request.
   *
   * @since 2.1.0
   * @since 4.4.0 Added the `$errors` parameter.
   *
   * @param WP_Error $errors A WP_Error object containing any errors generated
   *                         by using invalid credentials.
   */
  do_action( 'lostpassword_post', $errors );

  if ( $errors->get_error_code() )
    return $errors;

  if ( !$user_data ) {
    $errors->add( 'invalidcombo', __('Invalid username or email address, you may be using a different email address.', "wpep" ) );
    return $errors;
  }

  // Redefining user_login ensures we return the right case in the email.
  $user_login = $user_data->user_login;
  $user_email = $user_data->user_email;
  $key = get_password_reset_key( $user_data );

  if ( is_wp_error( $key ) ) {
    return $key;
  }

  $message = __('Someone has requested a password reset for the following account:') . "\r\n\r\n";
  $message .= network_home_url( '/' ) . "\r\n\r\n";
  $message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
  $message .= __('If this was a mistake, just ignore this email and nothing will happen.') . "\r\n\r\n";
  $message .= __('To reset your password, visit the following address:') . "\r\n\r\n";
  $message .= '<' . network_site_url("wp-login.php?action=rp&key=$key&login=" . rawurlencode($user_login), 'login') . ">\r\n";

  if ( is_multisite() ) {
    $blogname = get_network()->site_name;
  } else {
    /*
     * The blogname option is escaped with esc_html on the way into the database
     * in sanitize_option we want to reverse this for the plain text arena of emails.
     */
    $blogname = wp_specialchars_decode(get_option('blogname'), ENT_QUOTES);
  }

  /* translators: Password reset email subject. 1: Site name */
  $title = sprintf( __('[%s] Password Reset'), $blogname );

  /**
   * Filters the subject of the password reset email.
   *
   * @since 2.8.0
   * @since 4.4.0 Added the `$user_login` and `$user_data` parameters.
   *
   * @param string  $title      Default email title.
   * @param string  $user_login The username for the user.
   * @param WP_User $user_data  WP_User object.
   */
  $title = apply_filters( 'retrieve_password_title', $title, $user_login, $user_data );

  /**
   * Filters the message body of the password reset mail.
   *
   * @since 2.8.0
   * @since 4.1.0 Added `$user_login` and `$user_data` parameters.
   *
   * @param string  $message    Default mail message.
   * @param string  $key        The activation key.
   * @param string  $user_login The username for the user.
   * @param WP_User $user_data  WP_User object.
   */
  $message = apply_filters( 'retrieve_password_message', $message, $key, $user_login, $user_data );

  if ( $message && !wp_mail( $user_email, wp_specialchars_decode( $title ), $message ) )
    wp_die( __('The email could not be sent.') . "<br />\n" . __('Possible reason: your host may have disabled the mail() function.') );

  return true;
}