<?php

/**
 * @param $course_id
 * @return bool|int
 */
function wpep_course_assessments_required( $course_id ) {
  $assessments_required = get_post_meta( $course_id, 'wpep_course_assessments_required', true );

  if( $assessments_required === '' )
    return true;

  return intval( $assessments_required );
}

/**
 * @return array|mixed|string
 */
function wpep_course_has_bookmarks() {
  return wpep_get_setting_flag( 'enable-course-bookmarks' );
}

function wpep_course_type_from_id( $current_course_id ) {
  $current_course_type = get_post_meta( $current_course_id , 'course_type', true );

  $current_course_type = ( $current_course_type == false ? WPEP_COURSE_TYPE_EXECUTION_PLAN : $current_course_type );
  $current_course_type = apply_filters( 'wpep_single_content_course_type', $current_course_type, $current_course_id );

  return $current_course_type;
}

/**
 * @param bool|array $include
 * @param bool|array $exclude
 * @return array|string
 */
function wpep_course_layout_types( $include = false, $exclude = false ) {
  $layout_types = [
    WPEP_COURSE_TYPE_EXECUTION_PLAN          => __( 'Execution Plan', "wpep"),
    WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED => __( 'Execution Plan Unfolded', "wpep"),
    WPEP_COURSE_TYPE_STANDARD                => __( 'Standard', "wpep" ),
    WPEP_COURSE_TYPE_VIDEO                   => __( 'Video', "wpep" ),
    WPEP_COURSE_TYPE_ONE_OFF                 => __( 'One Off', "wpep" ),
    WPEP_COURSE_TYPE_LEARN_MAP               => __( 'Learn Map', "wpep" )
  ];

  $layout_types = apply_filters( 'wpep_course_layout_types', $layout_types );

  if( is_array( $exclude ) ) {
    foreach( $layout_types as $layout_type_key => $layout_type_label ) {
      if( !in_array( $layout_type_key, $exclude ) )
        continue;

      unset( $layout_types[ $layout_type_key ] );
    }
  } else if( is_array( $include ) ) {
    foreach( $layout_types as $layout_type_key => $layout_type_label ) {
      if( in_array( $layout_type_key, $include ) )
        continue;

      unset( $layout_types[ $layout_type_key ] );
    }
  }

  return $layout_types;
}

/**
 * @param bool|array $include
 * @param bool|array $exclude
 * @param bool|string $glue
 * @return array|string
 */
function wpep_course_layout_types_string( $include = false, $exclude = false, $glue = " " ) {
  return implode( $glue, array_keys( wpep_course_layout_types( $include, $exclude ) ) );
}

/**
 * @param $course_type
 * @return mixed|string
 */
function wpep_course_layout_type_label( $course_type ) {
  $layout_types = wpep_course_layout_types();

  return isset( $layout_types[ $course_type ] ) ? $layout_types[ $course_type ] : wpep_alias_to_label( $course_type );
}

/**
 * @param $course_id
 * @return string
 */
function wpep_course_thumbnail_url( $course_id ) {
  return get_post_meta( $course_id, 'wpep_thumb_image', true );
}

/**
 * @param $course_id
 * @return string
 */
function wpep_course_thumbnail_hover_url( $course_id ) {
  return get_post_meta( $course_id, 'wpep_thumb_image_hover', true );
}

/**
 * @param $course_id
 * @param bool $short_format
 * @return mixed|string
 */
function wpep_course_excerpt( $course_id, $short_format = false ) {
  $excerpt = get_post_meta( $course_id, 'wpep_course_excerpt', true );

  if( $excerpt == '' ) {
    $excerpt = get_post_meta( $course_id, 'wpep_course_description', true );
    $excerpt = strip_shortcodes( $excerpt );
    $excerpt = strip_tags( $excerpt );
    $excerpt = wp_trim_words( $excerpt, 15 );
  } else {
    if( $short_format ) {
      $short_excerpt = wp_trim_words( $excerpt, 20 );

      if( $excerpt != $short_excerpt )
        $excerpt = $short_excerpt . '...';
    } else {
      $excerpt = do_shortcode( $excerpt );
    }
  }

  return $excerpt;
}

/**
 * @param $course_id
 * @param bool|null $read_only
 * @return int
 */
function wpep_course_has_progress_bar( $course_id, $read_only = null ) {
  $read_only = ( $read_only === null ? wpep_post_read_only( $course_id ) : $read_only );

  if( $read_only != 1 ) {
    $show_progress_bar = get_post_meta( $course_id, 'wpep_show_progress_bar', true );

    if( $show_progress_bar == '' || $show_progress_bar == 'default' ) {
      $show_progress_bar = wpep_get_setting_flag( 'opt-course-percentage-bar' );
    } else {
      $show_progress_bar = ( $show_progress_bar == 'show' ? 1 : 0 );
    }
  } else {
    $show_progress_bar = 0;
  }

  return apply_filters( 'wpep_single_course_has_progress_bar', $show_progress_bar );
}

/**
 * @param $lesson_id
 * @param bool $course_id
 * @param string $slug
 * @return string
 */
function wpep_course_lesson_permalink( $lesson_id, $course_id = false, $slug = 'lesson' ) {
  if( $course_id == false )
    $course_id = get_the_ID();

  return wpep_permalink_extend( get_permalink( $course_id ), $slug, $lesson_id );
}

/**
 * @param $id
 * @return int
 */
function wpep_get_prerequisite_mode( $id ) {
  $types = wpep_get_prerequisite_modes();

  return ( isset( $types[ $id ] ) ? $types[ $id ] : ( isset( $types[WPEP_PREREQUISITE_MODE_NOTIFY] ) ? $types[WPEP_PREREQUISITE_MODE_NOTIFY] : 0 ) );
}

/**
 * @return array
 */
function wpep_get_prerequisite_modes() {
  return [
    WPEP_PREREQUISITE_MODE_NOTIFY  => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_NOTIFY,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "Recommend Prerequisite(s) Completion.", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => true,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 0,
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ],
    WPEP_PREREQUISITE_MODE_REQUIRE_ONE => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_REQUIRE_ONE,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "Student must complete 1 Prerequisite", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => false,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 1,
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ],
    WPEP_PREREQUISITE_MODE_REQUIRE_ONE_NOTIFY => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_REQUIRE_ONE_NOTIFY,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "Student must complete 1 Prerequisite before being suggested to complete other Prerequisites", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => true,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 1,
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ],
    WPEP_PREREQUISITE_MODE_REQUIRE_ALL => [
      WPEP_PREREQUISITE_FIELD_ID                     => WPEP_PREREQUISITE_MODE_REQUIRE_ALL,
      WPEP_PREREQUISITE_FIELD_NAME                   => __( "The student must complete all the prerequisites.", "wpep" ),
      WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION      => false,
      WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT => 'all',
      WPEP_PREREQUISITE_FIELD_SHOW_NEXT_COURSES      => true
    ]
  ];
}

/**
 * @return array
 */
function wpep_get_prerequisite_mode_list() {
  return wpep_utility_map_to_array_assoc( wpep_get_prerequisite_modes(), 'id', 'name' );
}

/**
 * @return array
 */
function wpep_course_type_member_magnet_integrated() {
  return wpep_get_setting_flag( 'enable-post-member-magnet-interaction' ) ? [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP ] : [];
}