<?php

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

/**
 * @return WPEP_Controller
 */
function wpep_controller() {
  return WPEP_Controller::instance();
}

/**
 * @param $option_name
 * @param string $option_default
 * @return string|array
 */
function wpep_get_setting( $option_name, $option_default = '' ) {
  return WPEP_Settings::instance()->get_option( $option_name, $option_default );
}

/**
 * @param $option_name
 * @return string
 */
function wpep_get_setting_translated( $option_name ) {
  return WPEP_Settings::instance()->get_option_translated( $option_name );
}

/**
 * @param $option_name
 * @return string
 */
function wpep_get_setting_flag( $option_name ) {
  return intval( WPEP_Settings::instance()->get_option( $option_name ) );
}

/**
 * @param $option_name
 * @param $option_value
 * @return bool
 */
function wpep_set_setting( $option_name, $option_value ) {
  return WPEP_Settings::instance()->update_settings( [
    $option_name => $option_value
  ] );
}

/**
 * @param $template_name
 * @param array $params
 * @return string
 */
function wpep_template_get( $template_name, $params = []) {
  return WPEP_Controller::instance()->template->get_template( $template_name, $params );
}

/**
 * @param $template_name
 * @param array $params
 */
function wpep_template_load( $template_name, $params = []) {
  WPEP_Controller::instance()->template->load_template( $template_name, $params );
}

/**
 * WPEP Proxy of WordPress get_header()
 */
function wpep_template_header() {
  WPEP_Controller::instance()->template->get_header();
}

/**
 * WPEP Proxy of WordPress get_footer()
 */
function wpep_template_footer() {
  WPEP_Controller::instance()->template->get_footer();
}

function wpep_has_sid_support() {
  return wpep_get_setting_flag( 'enable-sid-support' );
}

function wpep_has_embed_support() {
  return apply_filters( "wpep_has_embed_support_enabled", false );
}

function wpep_has_engagement_embed_support() {
  return wpep_has_embed_support() && wpep_get_setting_flag( 'user-content-embed-box-enabled' );
}

/**
 * @param $tracking_code
 * @param null|int $tracking_id
 * @return string
 */
function wpep_prepare_tracking_code( $tracking_code, $tracking_id = null ) {
  if( !is_string( $tracking_code ) || $tracking_code == '' )
    return '';

  $tracking_id = ( $tracking_id === null ? rand( 100, 1000 ) : $tracking_id );

  return '<div id="wpep-tracking-code-' . $tracking_id . '" class="wpep-tracking-code">' . $tracking_code . '</div>';
}

/**
 * @param array $data
 * @return string
 */
function wpep_breadcrumb( $data = [] ) {
  $response = '';

  if( empty( $data ) )
    return $response;

  $response .= '<div class="wpep-breadcrumb-container">';
  $response .=    '<ul class="wpep-breadcrumb">';

  foreach( $data as $link => $label ) {
    $response .= '<li>';

    if( $link == false )
      $response .=  '<span>' . $label . '</span>';
    else
      $response .=  '<a href="' . $link . '">&laquo; ' . $label . '</a>';
    $response .= '</li>';
  }

  $response .=    '</ul>';
  $response .=    '<div class="clear"></div>';
  $response .= '</div>';

  return $response;
}

function wpep_frontend_button( $link, $attributes = [] ) {
  $text           = isset($attributes['text']) ? $attributes['text'] : $link;
  $position       = isset($attributes['position']) ? $attributes['position'] : 'left';
  $force_download = isset($attributes['force_download']) ? $attributes['force_download'] : wpep_string_ends_with( $link, [ 'mp3', 'pdf', 'doc', 'mov', 'mp4' ] );
  $download_name  = isset($attributes['download_name'])  ? $attributes['download_name'] : basename( $link );

  $link = ( strpos( $link, "http" ) !== 0 ? 'http://' . $link : $link );

  $style = '';

  if( isset( $attributes['color'] ) )
    $style .= 'color : ' . $attributes['color'] . ';';

  if( isset( $attributes['background_color'] ) )
    $style .= 'background-color : ' . $attributes['background_color'] . ';';

  $response = '';

  $response .= '<p class="wpep-link-container wpep-link-container-' . $position . '">';
  $response .=    '<a href="' . $link . '" ';
  $response .=        ( $force_download ? ' download="' . $download_name . '" ' : '' );
  $response .=        ( $style != '' ? ' style="' . $style . '" ' : '' );
  $response .=     '>';
  $response .=      $text;
  $response .=    '</a>';
  $response .= '</p>';

  return $response;
}

/**
 * @param int $progress_percent
 * @return string
 */
function wpep_progress_bar( $progress_percent = 0 ) {
  $response = '';
  $response .= '<div class="wpep-progress-bar">';
  $response .=    '<div class="wpep-progress-bar-inner" role="progressbar" aria-valuenow="' . $progress_percent . '" aria-valuemin="0" aria-valuemax="100" style="width: ' . $progress_percent . '%"></div>';
  $response .= '</div>';

  return $response;
}

/**
 * @param $current_step
 * @param $step_count
 * @return string
 */
function wpep_navigation_bar( $current_step, $step_count ) {
  $response = '';
  $response .= '<div class="wpep-navigation-bar-container">';
  $response .=    '<ul class="wpep-navigation-bar wpep-navigation-bar-items-' . $step_count . ' wpep-navigation-bar-current-item-' . $current_step . '">';

  for( $i = 1; $i <= $step_count; $i++ ) {
    $class  = 'wpep-navigation-bar-point';
    $class .= ( $i < $current_step ? ' wpep-navigation-bar-point-passed' : '' );
    $class .= ( $i == $current_step ? ' wpep-navigation-bar-point-current' : '' );

    $response .=    '<li class="' . $class . '">' . $i . '</li>';
  }

  $response .=    '</ul>';
  $response .= '</div>';

  return $response;
}

/**
 * @param string|bool $title
 * @param string $container_class
 * @param string $span_class
 * @return string
 */
function wpep_spinner_loader( $title = false, $container_class = 'wpep-spinner-loader-container', $container_tag = 'div' ) {
  $content = '';

  $content  .= '<' . $container_tag . ' class="' . $container_class . '">';

  if( $title != false )
    $content .=       '<h2>' . $title . '</h2>';

  $content .=       '<span class="wpep-spinner-loader wpep-loading-container">' . __( "Loading...", "wpep" ) . '</span>';

  $content .= '</' . $container_tag . '>';

  return $content;
}

function wpep_is_rtl() {
  return is_rtl() || wpep_get_setting_flag( 'enforce-rtl-support' ) == true;
}

/**
 * @param $content
 * @param $id
 * @param bool $field_name
 * @param $editor_args
 * @return bool | true if it uses the wp_editor
 */
function wpep_editor( $content, $id, $field_name = false, $editor_args = [] ) {
  $editor_mode = WPEP_Settings::instance()->get_option( 'wpep-editor-setup' );
  $field_name  = ( $field_name == false ? $id : $field_name );

  if( $editor_mode == 'textarea' ) {
    echo '<textarea id="' . $id . '" 
                    class="wpep-editor wpep-editor-setup-textarea"
                    style="width:100%;max-width:960px;min-height:200px;"
                    name="' . $field_name . '">' . $content . '</textarea>';

    return false;
  }

  if( $editor_mode == 'medium-editor' ) {
    echo '<textarea id="' . $id . '" 
                    class="wpep-editor wpep-editor-setup-medium-editor"
                    style="width:100%;max-width:960px;min-height:200px;"
                    name="' . $field_name . '"
                    placeholder="' . __( "Start typing", "wpep" ) . '">' . wpautop( $content ) . '</textarea>';

    return false;
  }

  wpep_editor_wp( $content, $id, $field_name, $editor_args );
  
  return true;
}


/**
 * @param $content
 * @param $id
 * @param bool $field_name
 * @param $wp_editor_args
 * @return bool | true if it uses the wp_editor
 */
function wpep_editor_wp( $content, $id, $field_name = false, $wp_editor_args = [] ) {
  $field_name  = ( $field_name == false ? $id : $field_name );
  $editor_mode = WPEP_Settings::instance()->get_option( 'wpep-editor-setup' );

  if( !isset( $wp_editor_args['textarea_name'] ) )
    $wp_editor_args['textarea_name'] = $field_name;

  if( !isset( $wp_editor_args['textarea_rows'] ) )
    $wp_editor_args['textarea_rows'] = 10;

  if( $editor_mode == 'visual' ) {
    $wp_editor_args['quicktags']      = [];
    $wp_editor_args['default_editor'] = 'tinymce';
  }

  $wp_editor_args['editor_class'] = ( isset( $wp_editor_args['editor_class'] ) ? $wp_editor_args['editor_class'] . ' ' : '' ) . 'wpep-editor wpep-editor-wp-editor wpep-editor-setup-' . $editor_mode;

  wp_editor( $content, $id, $wp_editor_args );

  return true;
}

if( !function_exists( 'wpep_admin_url_statistics' ) ) {
  function wpep_admin_url_statistics( $params = [] ) {
    $url = admin_url( 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-statistics' );

    if( is_array( $params ) ) {
      foreach( $params as $key => $value )
        $url = add_query_arg( $key, $value, $url );

    } else {
      $url .= $params;
    }

    return apply_filters( 'wpep_administration_statistics_url', $url, $params );
  }
}

if( !function_exists( 'wpep_admin_url_platform_health' ) ) {
  function wpep_admin_url_platform_health( $params = [] ) {
    $url = admin_url( 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-platform-health' );

    if( is_array( $params ) ) {
      foreach( $params as $key => $value )
        $url = add_query_arg( $key, $value, $url );

    } else {
      $url .= $params;
    }

    return apply_filters( 'wpep_administration_platform_health_url', $url, $params );
  }
}

/**
 * @param $title
 * @param array $head_row_data
 * @param array $body_row_list
 * @param bool $display_download_button
 * @param null $pagination_current
 * @param null $pagination_total
 * @param string $table_class
 */
function wpep_admin_display_table(
  $title,
  $head_row_data = [],
  $body_row_list = [],
  $display_download_button = false,
  $pagination_current = null,
  $pagination_total   = null,
  $table_class        = 'wp-list-table widefat fixed striped wpep-statistics-table'
) {
  echo '<h2 class="wpep-statistics-title' . ( is_array( $title ) ? ' nav-tab-wrapper' : '' ) . '">';

  if( is_array( $title )  ) {
    foreach( $title as $tab_link => $tab_name ) {
      if( $tab_link == false ) {
        echo '<a class="nav-tab nav-tab-active" href="#">' . $tab_name . '</a>';
        continue;
      }

      echo '<a class="nav-tab" href="' . $tab_link . '">' . $tab_name . '</a>';
      continue;
    }
  } else {
    echo    $title;
  }

  echo    ( $display_download_button && !empty( $body_row_list ) ? '<form method="POST"><input type="submit" class="button button-primary" name="wpep_download_statistics" value="' . __( "Download", "wpep" ) . '"></form>' : '' );
  echo '</h2>';

  if( empty( $body_row_list ) ) {
    echo '<div class="wpep-notification wpep-notification-warning">';
    echo    '<p>' . __( "There are no statistics available right now, please check back later.", "wpep" ) . '</p>';
    echo '</div>';

    return;
  }

  echo '<table class="' . $table_class . '">';
  echo    '<thead>';
  echo      '<tr>';

  foreach( $head_row_data as $current_entry_column ) {
    if( !is_array( $current_entry_column ) )
      $current_entry_column = [ 'value' => $current_entry_column ];

    echo    '<th class="' . ( isset( $current_entry_column['type'] ) ? $current_entry_column['type'] : 'normal' ) . '"' .
                ( isset( $current_entry_column['style'] ) ? 'style="' . $current_entry_column['style'] . '" ' : '' ) .
                ' >' .
                $current_entry_column['value'] .
            '</th>';
  }

  echo      '</tr>';
  echo    '<tbody>';


  foreach( $body_row_list as $current_entry_row ) {
    echo '<tr>';

    foreach( $head_row_data as $current_entry_index => $current_entry_title ) {
      if( !isset( $current_entry_row[$current_entry_index] ) ) {
        echo '<td>-</td>';

        return;
      }

      $current_entry_column = $current_entry_row[$current_entry_index];

      if( !is_array( $current_entry_column ) )
        $current_entry_column = [ 'value' => $current_entry_column ];

      echo    '<td class="' . ( isset( $current_entry_column['type'] ) ? $current_entry_column['type'] : 'normal' ) . '" ' .
                ( isset( $current_entry_column['style'] ) ? 'style="' . $current_entry_column['style'] . '" ' : '' ) .
                ' >' .
        $current_entry_column['value'] .
        '</td>';
    }

    echo '</tr>';
  }

  echo    '</tbody>';
  echo '</table>';

  if( $pagination_current === null || $pagination_total === null )
    return;

  echo '<div class="wpep-pagination-links">';

  echo paginate_links( [
    'base'    => 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-statistics%_%',
    'format'  => '&pagination=%#%',
    'current' => $pagination_current,
    'total'   => $pagination_total
  ] );

  echo '</div>';
}

function wpep_get_external_page_embed_copy_button(
  $post_id,
  $permalink = false,
  $class     = 'button button-primary',
  $button_text        = '',
  $button_text_copied = ''
) {
  $permalink          = ( $permalink === false ? get_permalink( $post_id ) : $permalink );
  $button_text        = ( $button_text === ''  ? __( "Copy Embed Code", "wpep" ) : $button_text );
  $button_text_copied = ( $button_text_copied === '' ? __( "Copied !", "wpep" ) : $button_text_copied );

  return '<a class="' . $class . '"
               data-wpep-embed-id="wpep-embed-' . $post_id . '"
               data-wpep-embed-link-copy="' . add_query_arg( 'wpep-embed', 'true', $permalink ) . '"
               data-wpep-embed-link-copied="' . $button_text_copied . '">' . $button_text . '</a>';
}

/**
 * This is not by default used on the frontend, and the option Enforce Ordering is required to be checked.
 * @param $term_list
 * @return mixed
 */
function wpep_term_list_ordering( $term_list ) {
  if( !is_array( $term_list ) )
    return $term_list;

  if( empty( $term_list ) )
    return $term_list;

  foreach( $term_list as $term_index => $term )
    $term_list[$term_index]->wpep_order = intval( get_term_meta( $term->term_id, 'wpep_order', true ) );

  usort( $term_list, function( $a, $b ) {
    if ( $a->wpep_order == $b->wpep_order )
      return 0;

    return ( $a->wpep_order < $b->wpep_order ) ? -1 : 1;
  });

  return $term_list;
}

/**
 * @return array
 */
function wpep_toolkit_list_map() {
  $response = [
    'wpep_elements' => [
      'type'        => 'menu',
      'name'        => __( "Elements", "wpep" ),
      'items'       => [
        'wpep_notification' => [
          'type'        => 'builder',
          'name'        => __( "Notification", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_notification', [ 'wpep_admin_shortcode_builder_wpep_notification_options' ], 0 )
        ],
        'wpep_link'         => [
          'type'        => 'builder',
          'name'        => __( "Link Button", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_link', [ 'wpep_admin_shortcode_builder_wpep_link_options' ], 0 )
        ],
        'wpep_video' => [
          'type'        => 'plain',
          'name'        => __( "Video", "wpep" ),
          'before'      => '[wpep_video]',
          'placeholder' => __( "video-url-goes-here", "wpep" ),
          'after'       => '[/wpep_video]'
        ],
        'wpep_hero_section' => [
          'type'        => 'plain',
          'name'        => __( "Hero Section", "wpep" ),
          'before'      => '[wpep_hero_section]',
          'placeholder' => __( "Hero Section Content", "wpep" ),
          'after'       => '[/wpep_hero_section]'
        ]
      ]
    ],
    'wpep_index'             => [
      'type'        => 'builder',
      'name'        => __( "Index Page", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_index', [ 'wpep_admin_shortcode_builder_wpep_index_options' ] )
    ],
    'wpep_course'            => [
      'type'        => 'builder',
      'name'        => __( "Course Embed", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_course', [ 'wpep_admin_shortcode_builder_wpep_course_options' ] )
    ],
    'wpep_course_grid' => [
      'type'        => 'builder',
      'name'        => __( "Course 3D Grid", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_course_grid', [ 'wpep_admin_shortcode_builder_wpep_course_grid_options' ] )
    ],
    'wpep_profile_section' => [
      'type'        => 'builder',
      'name'        => __( "User Profile Page", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_profile', [] )
    ],
    'wpep_search_section' => [
      'type'        => 'menu',
      'name'        => 'Search Bar',
      'items'       => [
        'wpep_search_content' => [
          'type'        => 'plain',
          'name'        => __( "Content", "wpep" ),
          'before'      => '[wpep_search]'
        ],
        'wpep_search_courses' => [
          'type'        => 'plain',
          'name'        => __( "Courses", "wpep" ),
          'before'      => '[wpep_search post_type="' . WPEP_POST_TYPE_COURSE . '"]'
        ],
        'wpep_search_ebook' => [
          'type'        => 'plain',
          'name'        => __( "E-Books", "wpep" ),
          'before'      => '[wpep_search post_type="' . WPEP_POST_TYPE_E_BOOK . '"]'
        ],
        'wpep_search_offer' => [
          'type'        => 'plain',
          'name'        => __( "Offers", "wpep" ),
          'before'      => '[wpep_search post_type="' . WPEP_POST_TYPE_OFFER . '"]'
        ],
        'wpep_search_video' => [
          'type'        => 'plain',
          'name'        => __( "Videos", "wpep" ),
          'before'      => '[wpep_search post_type="' . WPEP_POST_TYPE_VIDEO . '"]'
        ],
        'wpep_search_advanced' => [
          'type'        => 'builder',
          'name'        => __( "Advanced Settings", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_search', [ 'wpep_admin_shortcode_builder_wpep_search_advanced_options' ] )
        ]
      ]
    ],
    'wpep_misc'     => [
      'type'  => 'menu',
      'name'  => __( "Miscellaneous", "wpep" ),
      'items' => [
        'wpep_center'            => [
          'type'        => 'plain',
          'name'        => __( "Content Center", "wpep" ),
          'before'      => '[wpep_center]',
          'placeholder' => __( "Content that will be centered", "wpep" ),
          'after'       => '[/wpep_center]'
        ],
        'wpep_profile_authentication'     => [
          'type'        => 'plain',
          'name'        => __( "Authentication Form", "wpep" ),
          'before'      => '[wpep_profile_authentication]',
          'placeholder' => "",
          'after'       => '[/wpep_profile_authentication]'
        ],
        'wpep_lesson_content' => [
          'type'        => 'builder',
          'name'        => __( "Lesson Content", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_lesson_content', [ 'wpep_admin_shortcode_builder_wpep_lesson_content_options' ] )
        ],
        'wpep_user_is_logged_in' => [
          'type'        => 'plain',
          'name'        => __( "User Logged in", "wpep" ),
          'before'      => '[wpep_user_is_logged_in]',
          'placeholder' => __( "Content visible for logged in users", "wpep" ),
          'after'       => '[/wpep_user_is_logged_in]'
        ],
        'wpep_user_is_logged_out' => [
          'type'        => 'plain',
          'name'        => __( "User Logged Out", "wpep" ),
          'before'      => '[wpep_user_not_logged_in]',
          'placeholder' => __( "Content visible for logged out users", "wpep" ),
          'after'       => '[/wpep_user_not_logged_in]'
        ],
        'wpep_course_is_completed' => [
          'type'        => 'builder',
          'name'        => __( "Course(s) Is Completed", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_course_is_completed', [ 'wpep_admin_shortcode_builder_wpep_course_ids_selection_options' ] )
        ],
        'wpep_course_not_completed' => [
          'type'        => 'builder',
          'name'        => __( "Course(s) Not Completed", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_course_not_completed', [ 'wpep_admin_shortcode_builder_wpep_course_ids_selection_options' ] )
        ]
      ]
    ]
  ];

  if( wpep_public_profiles_enabled() )
    $response[ 'wpep_misc' ][ 'items' ][ 'wpep_public_profile_link' ] = [
      'type'        => 'plain',
      'name'        => __( "Public Profile Link", "wpep" ),
      'before'      => '[wpep_public_profile_link]',
      'placeholder' => __( "Link Name", "wpep" ),
      'after'       => '[/wpep_public_profile_link]'
    ];

  return apply_filters( 'wpep_toolkit_list_items_map', $response );
}

/**
 * @param $tag
 * @param array $options
 * @param int $layout_options
 * @return string
 */
function wpep_toolkit_list_item_url( $tag, $options = [], $layout_options = 1 ) {
  $url = wpep_admin_shortcode_builder_url( $tag, $options );
  $url = add_query_arg( 'wpep-sb-init', 0, $url );
  $url = add_query_arg( 'wpep-sb-button-text', urlencode( __( "Use ShortCode", "wpep" ) ), $url );
  $url = add_query_arg( 'wpep-sb-button-description', urlencode( __( "The shortcode will be displayed in the editor", "wpep" ) ) , $url );
  $url = add_query_arg( 'wpep-sb-layout-options', intval( $layout_options ) , $url );

  return $url;
}

function wpep_get_theme_list_as_entity_layout_option() {
  $theme_list = WPEP_Controller::instance()->templateStyle->get_theme_list();

  $response = [
    ''  =>  sprintf( __( "Default Theme - %s", "wpep" ), $theme_list[ WPEP_Controller::instance()->templateStyle->theme ]['name'] )
  ];

  foreach( $theme_list as $theme_key => $theme_information )
    $response[ $theme_key ] = $theme_information['name'];

  return $response;
}

/**
 * @return array
 */
function wpep_get_custom_style_old_files() {
  $files = array_diff( scandir( wpep_get_upload_dir() ) , [ '.', '..' ] );

  foreach( $files as $file_index => $file )
    if( strpos( $file, 'custom-style-' . WPEP_VERSION ) === 0
        || strpos( $file, 'custom-style' ) !== 0 )
      unset( $files[$file_index] );

  return $files;
}

/**
 * @param string $prefix
 * @return bool|string
 */
function wpep_get_custom_style_file( $prefix = 'baseurl' ) {
  if( !file_exists( wpep_get_upload_dir() . '/' . wpep_get_custom_style_file_name() ) ) {
    $file_exists = wpep_refresh_custom_style_file();

    if( !$file_exists )
      return false;
  }

  return wpep_get_upload_dir( $prefix ) . '/' . wpep_get_custom_style_file_name();
}

/**
 * @param bool $wpep_theme
 * @return bool|string
 */
function wpep_refresh_custom_style_file( $wpep_theme = false ) {
  if( $wpep_theme === true ) {
    foreach( WPEP_Controller::instance()->templateStyle->get_theme_list() as $theme_key => $theme_information )
      wpep_refresh_custom_style_file( $theme_key );

    return true;
  }

  $wpep_upload_dir = wpep_get_upload_dir();

  if( !is_dir( $wpep_upload_dir ) )
    mkdir($wpep_upload_dir, 0777, true);

  $file_path = $wpep_upload_dir . '/' . wpep_get_custom_style_file_name( $wpep_theme );

  if( file_exists( $file_path ) )
    unlink( $file_path );

  file_put_contents( $file_path, wpep_controller()->templateStyle->get_custom_style( $wpep_theme ) );

  if( !file_exists( $file_path ) )
    return false;

  return $file_path;
}

/**
 * @param string $index
 * @return string
 */
function wpep_get_upload_dir( $index = 'basedir') {
  $upload_dir = wp_upload_dir();

  return $upload_dir[ $index ] . '/wpep';
}

/**
 * @param bool|string $wpep_theme
 * @return string
 */
function wpep_get_custom_style_file_name( $wpep_theme = false ) {
  $wpep_theme = ( $wpep_theme === false ? WPEP_Controller::instance()->templateStyle->theme : $wpep_theme );

  return 'custom-style-' . WPEP_VERSION . '-' . $wpep_theme . '.css';
}

function wpep_custom_style_format_legacy_rules( $custom_css ) {
  $map = [
    '#course-grid-category-list'  => '.course-grid-category-list',
    '#course-grid'                => '.course-grid-container',
    '#course-primary'             => '.course-primary-container',
    '#course-content'             => '.course-primary-content',
    '#wpep-secondary'             => '.course-sidebar-content',
    '#course-grid-main-head'      => '.course-grid-container-header',
    '#course-grid-sub-head'       => '.course-grid-container-content'
  ];

  foreach( $map as $key => $value )
    $custom_css = str_replace( $key, $value, $custom_css );

  return $custom_css;
}

function wpep_flag_setting_field_options() {
  return [
    0         => __( 'Disabled', "wpep" ),
    1         => __( 'Enabled', "wpep" ),
  ];
}